import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    EventEmitter,
    inject,
    OnDestroy,
    OnInit,
    Output
} from '@angular/core';
import { TimezoneConfiguration, TimezoneService } from '../../../../services/timezone.service';
import { Subscription } from 'rxjs';
import {
    AlertComponent,
    BorderDirective,
    ButtonCloseDirective,
    ButtonGroupComponent,
    ButtonToolbarComponent,
    CardBodyComponent,
    CardComponent,
    CardHeaderComponent,
    ColComponent,
    ModalBodyComponent,
    ModalComponent,
    ModalFooterComponent,
    ModalHeaderComponent,
    ModalService,
    ModalTitleDirective,
    RowComponent,
    TooltipDirective
} from '@coreui/angular';
import { HostsService } from '../../hosts.service';
import { PermissionsService } from '../../../../permissions/permissions.service';
import { HostProcessCheckResultItem } from '../../../../services/external-commands.service';
import { ExternalCommandsEnum } from '../../../../enums/external-commands.enum';
import { HostBrowserDowntime } from '../../../downtimes/downtimes.interface';
import { CancelAllItem } from '../../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime.interface';
import { AsyncPipe, NgClass, NgIf } from '@angular/common';
import { BlockLoaderComponent } from '../../../../layouts/primeng/loading/block-loader/block-loader.component';
import {
    CancelHostdowntimeModalComponent
} from '../../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime-modal.component';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import {
    HostsDisableFlapdetectionModalComponent
} from '../../../../components/hosts/hosts-disable-flapdetection-modal/hosts-disable-flapdetection-modal.component';
import {
    HostsEnableFlapdetectionModalComponent
} from '../../../../components/hosts/hosts-enable-flapdetection-modal/hosts-enable-flapdetection-modal.component';
import { PermissionDirective } from '../../../../permissions/permission.directive';
import { TranslocoDirective, TranslocoPipe } from '@jsverse/transloco';
import { TrustAsHtmlPipe } from '../../../../pipes/trust-as-html.pipe';
import { XsButtonDirective } from '../../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { RouterLink } from '@angular/router';
import { HostsBrowserRoot, HoststatusBrowser } from '../../hosts.interface';
import { HostsBrowserModalService } from './hosts-browser-modal.service';
import {
    HostAcknowledgeModalComponent
} from '../../../../components/hosts/host-acknowledge-modal/host-acknowledge-modal.component';
import {
    HostsMaintenanceModalComponent
} from '../../../../components/hosts/hosts-maintenance-modal/hosts-maintenance-modal.component';
import {
    HostsProcessCheckresultModalComponent
} from '../../../../components/hosts/hosts-process-checkresult-modal/hosts-process-checkresult-modal.component';
import {
    ServiceResetChecktimeModalComponent
} from '../../../../components/services/service-reset-checktime-modal/service-reset-checktime-modal.component';
import { HostStatusNamePipe } from '../../../../pipes/host-status-name.pipe';
import {
    HostsBrowserServicesListComponent
} from '../hosts-browser-services-list/hosts-browser-services-list.component';
import { DELETE_SERVICE_TOKEN } from '../../../../tokens/delete-injection.token';
import { DowntimesService } from '../../../downtimes/downtimes.service';


@Component({
    selector: 'oitc-hosts-browser-modal',
    imports: [
        AlertComponent,
        BlockLoaderComponent,
        ButtonCloseDirective,
        ButtonGroupComponent,
        ButtonToolbarComponent,
        CancelHostdowntimeModalComponent,
        CardBodyComponent,
        CardComponent,
        CardHeaderComponent,
        ColComponent,
        FaIconComponent,
        HostsDisableFlapdetectionModalComponent,
        HostsEnableFlapdetectionModalComponent,
        ModalBodyComponent,
        ModalComponent,
        ModalFooterComponent,
        ModalHeaderComponent,
        ModalTitleDirective,
        NgIf,
        PermissionDirective,
        RowComponent,
        TranslocoDirective,
        TranslocoPipe,
        TrustAsHtmlPipe,
        XsButtonDirective,
        NgClass,
        TooltipDirective,
        BorderDirective,
        RouterLink,
        HostAcknowledgeModalComponent,
        HostsMaintenanceModalComponent,
        HostsProcessCheckresultModalComponent,
        ServiceResetChecktimeModalComponent,
        HostStatusNamePipe,
        AsyncPipe,
        HostsBrowserServicesListComponent,

    ],
    templateUrl: './hosts-browser-modal.component.html',
    styleUrl: './hosts-browser-modal.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: DowntimesService}, // Inject the DowntimesService into the CancelAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HostsBrowserModalComponent implements OnInit, OnDestroy {
    public hostUuid: string = '';

    @Output() completed: EventEmitter<boolean> = new EventEmitter<boolean>();

    public timezone!: TimezoneConfiguration;
    public result?: HostsBrowserRoot;
    public lastUpdated: Date = new Date(); // Used to tell child components to reload data

    public selectedItems: any[] = [];

    public cssClass: string = 'bg-unreachable';
    public textClass: string = 'unreachable';

    private subscriptions: Subscription = new Subscription();
    private readonly HostsService = inject(HostsService);
    private readonly TimezoneService = inject(TimezoneService);
    public readonly PermissionsService = inject(PermissionsService);
    private readonly modalService = inject(ModalService);
    private readonly HostsBrowserModalService = inject(HostsBrowserModalService);
    private readonly DowntimesService = inject(DowntimesService);
    private cdr = inject(ChangeDetectorRef);

    public hideModal() {
        this.modalService.toggle({
            show: false,
            id: 'hostsBrowserModal'
        });
    }

    private resetModal(): void {
        // Reset everything
        this.selectedItems = [];
        this.result = undefined;
        this.cdr.markForCheck();
    }

    public ngOnInit(): void {
        this.getUserTimezone();

        this.subscriptions.add(this.HostsBrowserModalService.hostUuid$.subscribe((hostUuid) => {
            this.hostUuid = hostUuid;
            this.resetModal();

            this.loadHost();
            this.cdr.markForCheck();

            // open modal
            // no idea why we need setTimeout here, but otherwise you can not open the modal on the dashboard
            // more than once
            setTimeout(() => {
                this.modalService.toggle({
                    show: true,
                    id: 'hostsBrowserModal'
                });
            }, 150);
        }));
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    private getUserTimezone() {
        this.subscriptions.add(this.TimezoneService.getTimezoneConfiguration().subscribe(data => {
            this.timezone = data;
            this.cdr.markForCheck();
        }));
    }

    public loadHost() {
        if (this.hostUuid !== '') {
            this.subscriptions.add(this.HostsService.getHostBrowser(this.hostUuid).subscribe((result) => {
                this.result = result;

                switch (result.hoststatus.current_state) {
                    case 0:
                        this.cssClass = 'bg-up';
                        this.textClass = 'up';
                        break;
                    case 1:
                        this.cssClass = 'bg-down';
                        this.textClass = 'down';
                        break;

                    default:
                        this.cssClass = 'bg-unreachable';
                        this.textClass = 'unreachable';
                        break;
                }

                this.lastUpdated = new Date();
                this.cdr.markForCheck();
            }));
        }
    }

    public resetChecktime(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.rescheduleHost,
            hostUuid: hoststatus.hostname,
            type: 'hostOnly',
        });

        this.modalService.toggle({
            show: true,
            id: 'serviceResetChecktimeModal'
        });
    }

    public toggleDowntimeModal(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitHostDowntime,
                hostUuid: hoststatus.hostname,
                start: 0,
                end: 0,
                author: this.result.username,
                comment: '',
                downtimetype: ''
            });

            this.modalService.toggle({
                show: true,
                id: 'hostMaintenanceModal',
            });
        }
    }

    public acknowledgeStatus(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitHoststateAck,
                hostUuid: hoststatus.hostname,
                hostAckType: 'hostOnly',
                author: this.result.username,
                comment: '',
                notify: true,
                sticky: 0
            });

            this.modalService.toggle({
                show: true,
                id: 'hostAcknowledgeModal',
            });
        }
    }

    public processCheckResult(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        const item: HostProcessCheckResultItem = {
            command: ExternalCommandsEnum.commitPassiveResult,
            maxCheckAttempts: Number(hoststatus.max_check_attempts),
            hostUuid: String(hoststatus.hostname),
            status_code: 0, // Will be overwritten with the modals default value
            plugin_output: '', // Will be overwritten with the modals default value
            forceHardstate: false, // Will be overwritten with the modals default value
            long_output: ''
        };

        this.selectedItems.push(item);

        this.modalService.toggle({
            show: true,
            id: 'hostProcessCheckresultModal',
        });
    }

    public enableFlapdetection(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableHostFlapdetection,
            hostUuid: hoststatus.hostname,
            condition: 1
        });

        this.modalService.toggle({
            show: true,
            id: 'hostEnableFlapdetectionModal',
        });
    }

    public disableFlapdetection(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableHostFlapdetection,
            hostUuid: hoststatus.hostname,
            condition: 0
        });

        this.modalService.toggle({
            show: true,
            id: 'hostDisableFlapdetectionModal',
        });
    }

    public toggleCancelDowntimeModal(hostDowntime: HostBrowserDowntime) {
        this.selectedItems = [];

        const item: CancelAllItem[] = [{
            id: hostDowntime.internal_downtime_id
        }];

        // Pass selection to the modal
        this.selectedItems = item;

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'cancelHostAllModal',
        });
    }


    public onToggleRescheduling(event: boolean) {
        // Host browser menu wants to toggle a rescheduling
        if (this.result) {
            this.resetChecktime(this.result.hoststatus);
        }
    }

    public onMassActionComplete(success: boolean) {
        this.completed.emit(success); // Pass the event to the parent component
        if (success) {
            this.loadHost();
        }
    }

    protected readonly Number = Number;
    protected readonly String = String;
    protected readonly Boolean = Boolean;
}
