import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { Subscription } from 'rxjs';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { ActivatedRoute, Router } from '@angular/router';
import { HistoryService } from '../../../history.service';
import { HostsService } from '../hosts.service';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { PermissionDirective } from '../../../permissions/permission.directive';
import {
    AlertComponent,
    BadgeComponent,
    BorderDirective,
    ButtonGroupComponent,
    ButtonToolbarComponent,
    CardBodyComponent,
    CardComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective,
    TooltipDirective
} from '@coreui/angular';
import { HostBrowserMenuConfig, HostsBrowserMenuComponent } from '../hosts-browser-menu/hosts-browser-menu.component';
import { AsyncPipe, NgClass, NgIf } from '@angular/common';
import { BrowserLoaderComponent } from '../../../layouts/primeng/loading/browser-loader/browser-loader.component';
import { HostsBrowserRoot, HoststatusBrowser } from '../hosts.interface';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { BackButtonDirective } from '../../../directives/back-button.directive';
import { HostBrowserTabs } from '../hosts.enum';
import { PermissionsService } from '../../../permissions/permissions.service';
import {
    HostAcknowledgeModalComponent
} from '../../../components/hosts/host-acknowledge-modal/host-acknowledge-modal.component';

import {
    HostsMaintenanceModalComponent
} from '../../../components/hosts/hosts-maintenance-modal/hosts-maintenance-modal.component';
import {
    ServiceResetChecktimeModalComponent
} from '../../../components/services/service-reset-checktime-modal/service-reset-checktime-modal.component';
import { ExternalCommandsEnum } from '../../../enums/external-commands.enum';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import {
    HostsProcessCheckresultModalComponent
} from '../../../components/hosts/hosts-process-checkresult-modal/hosts-process-checkresult-modal.component';
import { HostProcessCheckResultItem } from '../../../services/external-commands.service';
import {
    HostsEnableFlapdetectionModalComponent
} from '../../../components/hosts/hosts-enable-flapdetection-modal/hosts-enable-flapdetection-modal.component';
import {
    HostsDisableFlapdetectionModalComponent
} from '../../../components/hosts/hosts-disable-flapdetection-modal/hosts-disable-flapdetection-modal.component';
import { HostStatusNamePipe } from '../../../pipes/host-status-name.pipe';
import { HostBrowserDowntime } from '../../downtimes/downtimes.interface';
import { DowntimesService } from '../../downtimes/downtimes.service';
import { DELETE_SERVICE_TOKEN } from '../../../tokens/delete-injection.token';
import { TrustAsHtmlPipe } from '../../../pipes/trust-as-html.pipe';
import { AcknowledgementsService } from '../../acknowledgements/acknowledgements.service';

import { TitleService } from '../../../services/title.service';
import { CancelAllItem } from '../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime.interface';
import {
    CancelHostdowntimeModalComponent
} from '../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime-modal.component';
import { ObjectUuidComponent } from '../../../layouts/coreui/object-uuid/object-uuid.component';
import { HostsBrowserServicesListComponent } from './hosts-browser-services-list/hosts-browser-services-list.component';

@Component({
    selector: 'oitc-hosts-browser',
    imports: [
        TranslocoDirective,
        FaIconComponent,
        PermissionDirective,
        RowComponent,
        ColComponent,
        HostsBrowserMenuComponent,
        NgIf,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        CardBodyComponent,
        BrowserLoaderComponent,
        NavComponent,
        NavItemComponent,
        XsButtonDirective,
        BackButtonDirective,
        NgClass,
        ButtonGroupComponent,
        ButtonToolbarComponent,
        TranslocoPipe,
        TooltipDirective,
        HostAcknowledgeModalComponent,
        HostsMaintenanceModalComponent,
        ServiceResetChecktimeModalComponent,
        HostsProcessCheckresultModalComponent,
        HostsEnableFlapdetectionModalComponent,
        HostsDisableFlapdetectionModalComponent,
        BorderDirective,
        HostStatusNamePipe,
        AlertComponent,
        TrustAsHtmlPipe,
        TableDirective,
        BadgeComponent,
        AsyncPipe,
        CancelHostdowntimeModalComponent,
        ObjectUuidComponent,
        HostsBrowserServicesListComponent
    ],
    templateUrl: './hosts-browser.component.html',
    styleUrl: './hosts-browser.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: DowntimesService}, // Inject the DowntimesService into the CancelAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HostsBrowserComponent implements OnInit, OnDestroy {

    public uuid: string = '';

    public hostBrowserConfig?: HostBrowserMenuConfig;

    public result?: HostsBrowserRoot;
    public lastUpdated: Date = new Date(); // Used to tell child components to reload data

    public selectedTab: HostBrowserTabs = HostBrowserTabs.StatusInformation;
    public selectedItems: any[] = [];

    public cssClass: string = 'bg-unreachable';
    public textClass: string = 'unreachable';

    private subscriptions: Subscription = new Subscription();
    private HostsService = inject(HostsService);
    private readonly notyService = inject(NotyService);
    private router = inject(Router);
    private route = inject(ActivatedRoute);
    private readonly HistoryService: HistoryService = inject(HistoryService);
    private readonly TitleService: TitleService = inject(TitleService);
    private readonly TranslocoService: TranslocoService = inject(TranslocoService);
    public readonly PermissionsService = inject(PermissionsService);
    private SelectionServiceService: SelectionServiceService = inject(SelectionServiceService);
    private readonly modalService = inject(ModalService);
    private readonly DowntimesService = inject(DowntimesService);
    private readonly AcknowledgementsService = inject(AcknowledgementsService);
    private cdr = inject(ChangeDetectorRef);

    constructor() {
    }

    public ngOnInit(): void {
        this.uuid = String(this.route.snapshot.paramMap.get('uuid'));
        this.loadHost();


        this.route.queryParams.subscribe(params => {
            let selectedTab = params['selectedTab'] || undefined;
            if (selectedTab) {
                this.changeTab(selectedTab);
                this.cdr.markForCheck();
            }
        });

    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    public loadHost() {
        // Define the configuration for the HostBrowserMenuComponent because we know the hostId now
        this.hostBrowserConfig = {
            hostUuid: this.uuid,
            showReschedulingButton: true,
            rescheduleCallback: () => {
                console.log('implement callback')
            },
            showBackButton: false
        };

        this.subscriptions.add(this.HostsService.getHostBrowser(this.uuid).subscribe((result) => {
            this.cdr.markForCheck();
            this.result = result;

            switch (result.hoststatus.current_state) {
                case 0:
                    this.cssClass = 'bg-up';
                    this.textClass = 'up';
                    break;
                case 1:
                    this.cssClass = 'bg-down';
                    this.textClass = 'down';
                    break;

                default:
                    this.cssClass = 'bg-unreachable';
                    this.textClass = 'unreachable';
                    break;
            }

            this.lastUpdated = new Date();

            // Update the title.
            let newTitle: string = this.result.hoststatus.host_mapping.display_name ?? this.TranslocoService.translate('Host');
            this.TitleService.setTitle(`${newTitle} | ` + this.TranslocoService.translate('Host Browser'));

        }));
    }

    public changeTab(newTab: HostBrowserTabs): void {
        this.selectedTab = newTab;
    }

    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadHost();
        }
    }

    public resetChecktime(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.rescheduleHost,
            hostUuid: hoststatus.hostname,
            type: 'hostOnly',
        });

        this.modalService.toggle({
            show: true,
            id: 'serviceResetChecktimeModal'
        });
    }

    public toggleDowntimeModal(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitHostDowntime,
                hostUuid: hoststatus.hostname,
                start: 0,
                end: 0,
                author: this.result.username,
                comment: '',
                downtimetype: ''
            });

            this.modalService.toggle({
                show: true,
                id: 'hostMaintenanceModal',
            });
        }
    }

    public acknowledgeStatus(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitHoststateAck,
                hostUuid: hoststatus.hostname,
                hostAckType: 'hostOnly',
                author: this.result.username,
                comment: '',
                notify: true,
                sticky: 0
            });

            this.modalService.toggle({
                show: true,
                id: 'hostAcknowledgeModal',
            });
        }
    }

    public processCheckResult(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        const item: HostProcessCheckResultItem = {
            command: ExternalCommandsEnum.commitPassiveResult,
            maxCheckAttempts: Number(hoststatus.max_check_attempts),
            hostUuid: String(hoststatus.hostname),
            status_code: 0, // Will be overwritten with the modals default value
            plugin_output: '', // Will be overwritten with the modals default value
            forceHardstate: false, // Will be overwritten with the modals default value
            long_output: ''
        };

        this.selectedItems.push(item);

        this.modalService.toggle({
            show: true,
            id: 'hostProcessCheckresultModal',
        });
    }

    public enableFlapdetection(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableHostFlapdetection,
            hostUuid: hoststatus.hostname,
            condition: 1
        });

        this.modalService.toggle({
            show: true,
            id: 'hostEnableFlapdetectionModal',
        });
    }

    public disableFlapdetection(hoststatus: HoststatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableHostFlapdetection,
            hostUuid: hoststatus.hostname,
            condition: 0
        });

        this.modalService.toggle({
            show: true,
            id: 'hostDisableFlapdetectionModal',
        });
    }

    public toggleCancelDowntimeModal(hostDowntime: HostBrowserDowntime) {
        this.selectedItems = [];

        const item: CancelAllItem[] = [{
            id: hostDowntime.internal_downtime_id
        }];

        // Pass selection to the modal
        this.selectedItems = item;

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'cancelHostAllModal',
        });
    }


    public onToggleRescheduling(event: boolean) {
        // Host browser menu wants to toggle a rescheduling
        if (this.result) {
            this.resetChecktime(this.result.hoststatus);
        }
    }

    protected readonly HostBrowserTabs = HostBrowserTabs;
    protected readonly Number = Number;
    protected readonly String = String;
    protected readonly Boolean = Boolean;
}
