import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import { Subscription } from 'rxjs';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { NotyService } from '../../../layouts/coreui/noty.service';
import {
    CardBodyComponent,
    CardComponent,
    CardFooterComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ContainerComponent,
    DropdownComponent,
    DropdownItemDirective,
    DropdownMenuDirective,
    DropdownToggleDirective,
    FormCheckComponent,
    FormCheckInputDirective,
    FormCheckLabelDirective,
    FormControlDirective,
    FormDirective,
    InputGroupComponent,
    InputGroupTextDirective,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective,
    TooltipDirective
} from '@coreui/angular';
import { MatSort, MatSortHeader, Sort } from '@angular/material/sort';
import { PaginatorChangeEvent } from '../../../layouts/coreui/paginator/paginator.interface';
import { ExternalCommandsEnum } from '../../../enums/external-commands.enum';
import {
    ExternalCommandsService,
    ServiceAcknowledgeItem,
    ServiceDowntimeItem,
    ServiceResetItem
} from '../../../services/external-commands.service';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { DebounceDirective } from '../../../directives/debounce.directive';
import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { FormsModule } from '@angular/forms';
import { TableLoaderComponent } from '../../../layouts/primeng/loading/table-loader/table-loader.component';
import { AsyncPipe, NgIf } from '@angular/common';
import { ItemSelectComponent } from '../../../layouts/coreui/select-all/item-select/item-select.component';
import {
    AcknowledgementIconComponent
} from '../../acknowledgements/acknowledgement-icon/acknowledgement-icon.component';
import { DowntimeIconComponent } from '../../downtimes/downtime-icon/downtime-icon.component';
import { TrustAsHtmlPipe } from '../../../pipes/trust-as-html.pipe';
import {
    PaginateOrScrollComponent
} from '../../../layouts/coreui/paginator/paginate-or-scroll/paginate-or-scroll.component';
import { NoRecordsComponent } from '../../../layouts/coreui/no-records/no-records.component';
import { SelectAllComponent } from '../../../layouts/coreui/select-all/select-all.component';
import { PermissionsService } from '../../../permissions/permissions.service';
import { SelectionServiceService } from '../../../layouts/coreui/select-all/selection-service.service';
import { IndexPage } from '../../../pages.interface';
import {
    getDefaultServicesIndexParams,
    getServiceCurrentStateForApi,
    ServicesCurrentStateFilter,
    ServicesIndexParams,
    ServicesIndexRoot
} from '../services.interface';
import { ServicesService } from '../services.service';
import {
    ServicestatusIconComponent
} from '../../../components/services/servicestatus-icon/servicestatus-icon.component';
import {
    ServiceMaintenanceModalComponent
} from '../../../components/services/service-maintenance-modal/service-maintenance-modal.component';
import {
    ServiceAcknowledgeModalComponent
} from '../../../components/services/service-acknowledge-modal/service-acknowledge-modal.component';
import { HoststatusIconComponent } from '../../hosts/hoststatus-icon/hoststatus-icon.component';
import { PopoverGraphComponent } from '../../../components/popover-graph/popover-graph.component';
import { TimezoneConfiguration, TimezoneService } from '../../../services/timezone.service';

@Component({
    selector: 'oitc-services-index',
    imports: [
        CardFooterComponent,
        RouterLink,
        PermissionDirective,
        FaIconComponent,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        NavComponent,
        NavItemComponent,
        XsButtonDirective,
        CardBodyComponent,
        ContainerComponent,
        RowComponent,
        ColComponent,
        FormDirective,
        InputGroupComponent,
        InputGroupTextDirective,
        DebounceDirective,
        TranslocoPipe,
        FormsModule,
        FormControlDirective,
        FormCheckComponent,
        FormCheckInputDirective,
        FormCheckLabelDirective,
        TranslocoDirective,
        TableLoaderComponent,
        TableDirective,
        MatSort,
        MatSortHeader,
        NgIf,
        TooltipDirective,
        ItemSelectComponent,
        AsyncPipe,
        AcknowledgementIconComponent,
        DowntimeIconComponent,
        TrustAsHtmlPipe,
        PaginateOrScrollComponent,
        NoRecordsComponent,
        SelectAllComponent,
        DropdownComponent,
        DropdownToggleDirective,
        DropdownMenuDirective,
        DropdownItemDirective,
        ServicestatusIconComponent,
        ServiceMaintenanceModalComponent,
        ServiceAcknowledgeModalComponent,
        HoststatusIconComponent,
        PopoverGraphComponent
    ],
    templateUrl: './services-index.component.html',
    styleUrl: './services-index.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServicesIndexComponent implements OnInit, OnDestroy, IndexPage {
    public params: ServicesIndexParams = getDefaultServicesIndexParams();

    public currentStateFilter: ServicesCurrentStateFilter = {
        ok: false,
        warning: false,
        critical: false,
        unknown: false
    };
    public state_typesFilter = {
        soft: false,
        hard: false
    };
    public acknowledgementsFilter = {
        acknowledged: false,
        not_acknowledged: false
    };
    public downtimeFilter = {
        in_downtime: false,
        not_in_downtime: false
    };

    public services?: ServicesIndexRoot;
    public hideFilter: boolean = true;

    public selectedItems: any[] = [];

    public timezone!: TimezoneConfiguration;
    public userFullname: string = '';

    private readonly ServicesService = inject(ServicesService);
    private subscriptions: Subscription = new Subscription();
    public readonly PermissionsService = inject(PermissionsService);
    private readonly TimezoneService = inject(TimezoneService);
    public readonly route = inject(ActivatedRoute);
    public readonly router = inject(Router);
    private readonly TranslocoService: TranslocoService = inject(TranslocoService)
    private readonly notyService: NotyService = inject(NotyService);
    private SelectionServiceService: SelectionServiceService = inject(SelectionServiceService);
    private readonly ExternalCommandsService = inject(ExternalCommandsService);
    private readonly modalService = inject(ModalService);

    private cdr = inject(ChangeDetectorRef);

    public ngOnInit() {
        this.getUserTimezone();

        this.route.queryParams.subscribe(params => {
            let hostname = params['hostname'] || undefined;
            if (hostname) {
                this.params['filter[HostMapping.display_name]'] = hostname;
            }

            let servicename = params['servicename'] || undefined;
            if (servicename) {
                this.params['filter[ServiceMapping.display_name]'] = servicename;
            }

            let direction = params['direction'];
            if (direction && direction === 'asc' || direction === 'desc') {
                this.params.direction = direction;
            }

            let sort = params['sort'];
            if (sort) {
                this.params.sort = sort;
            }

            let inDowntime = params['inDowntime'] || undefined;
            if (inDowntime) {
                let isFilterInDowntime: boolean = inDowntime === 'true';
                this.downtimeFilter.in_downtime = isFilterInDowntime;
                this.downtimeFilter.not_in_downtime = !isFilterInDowntime;
            }

            let acknowledged = params['acknowledged'] || undefined;
            if (acknowledged) {
                let isFilterAcknowledged : boolean = acknowledged === 'true';
                this.acknowledgementsFilter.acknowledged = isFilterAcknowledged;
                this.acknowledgementsFilter.not_acknowledged = !isFilterAcknowledged;
            }

            let servicestate = params['servicestate'] || undefined;
            if (servicestate) {
                //first reset all states
                this.currentStateFilter.ok = false;
                this.currentStateFilter.warning = false;
                this.currentStateFilter.critical = false
                this.currentStateFilter.unknown = false
                servicestate = [].concat(servicestate); // make sure we always get an array
                servicestate.forEach((state: any) => {
                    switch (parseInt(state, 10)) {
                        case 0:
                            this.currentStateFilter.ok = true;
                            break;
                        case 1:
                            this.currentStateFilter.warning = true;
                            break;
                        case 2:
                            this.currentStateFilter.critical = true;
                            break;
                        case 3:
                            this.currentStateFilter.unknown = true;
                            break;
                        default:
                            break;
                    }
                });
            }

            // Process all query params first and then trigger the load function
            this.loadServices();
        });

    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    private getUserTimezone() {
        this.subscriptions.add(this.TimezoneService.getTimezoneConfiguration().subscribe(data => {
            this.timezone = data;
            this.cdr.markForCheck();
        }));
    }

    public loadServices() {
        this.SelectionServiceService.deselectAll();

        let hasBeenAcknowledged = '';
        let inDowntime = '';
        if (this.acknowledgementsFilter.acknowledged !== this.acknowledgementsFilter.not_acknowledged) {
            hasBeenAcknowledged = String(this.acknowledgementsFilter.acknowledged === true);
        }
        if (this.downtimeFilter.in_downtime !== this.downtimeFilter.not_in_downtime) {
            inDowntime = String(this.downtimeFilter.in_downtime === true);
        }

        let state_type: string = '';
        if (this.state_typesFilter.soft !== this.state_typesFilter.hard) {
            state_type = '0';
            if (this.state_typesFilter.hard) {
                state_type = '1';
            }
        }


        this.params['filter[Servicestatus.problem_has_been_acknowledged]'] = hasBeenAcknowledged;
        this.params['filter[Servicestatus.scheduled_downtime_depth]'] = inDowntime;
        this.params['filter[Servicestatus.is_hardstate]'] = state_type;
        this.params['filter[Servicestatus.current_state][]'] = getServiceCurrentStateForApi(this.currentStateFilter);

        this.subscriptions.add(this.ServicesService.getIndex(this.params)
            .subscribe((result) => {
                this.services = result;
                this.userFullname = result.username;
                this.cdr.markForCheck();
            })
        );
    }

    // Show or hide the filter
    // Called by (click) - no manual change detection required
    public toggleFilter() {
        this.hideFilter = !this.hideFilter;
    }

    // Called by (click) - no manual change detection required
    public problemsOnly() {
        this.resetFilter(false);

        this.currentStateFilter = {
            ok: false,
            warning: true,
            critical: true,
            unknown: true
        };
        this.acknowledgementsFilter = {
            acknowledged: false,
            not_acknowledged: true
        };
        this.downtimeFilter = {
            in_downtime: false,
            not_in_downtime: true
        };

        this.cdr.markForCheck();

        this.loadServices();

    }

    // Callback when a filter has changed
    public onFilterChange(event: any) {
        this.params.page = 1;
        this.loadServices();
    }

    // Callback when sort has changed
    public onSortChange(sort: Sort) {
        if (sort.direction) {
            this.params.sort = sort.active;
            this.params.direction = sort.direction;
            this.loadServices();
        }
    }

    // Callback for Paginator or Scroll Index Component
    public onPaginatorChange(change: PaginatorChangeEvent): void {
        this.params.page = change.page;
        this.params.scroll = change.scroll;
        this.loadServices();
    }

    public resetFilter(doReload: boolean = true) {
        // Reset query parameters
        this.router.navigate([], {
            queryParams: {},
            queryParamsHandling: "",
        });

        this.params = getDefaultServicesIndexParams();
        this.currentStateFilter = {
            ok: false,
            warning: false,
            critical: false,
            unknown: false
        };
        this.state_typesFilter = {
            soft: false,
            hard: false
        };
        this.acknowledgementsFilter = {
            acknowledged: false,
            not_acknowledged: false
        };
        this.downtimeFilter = {
            in_downtime: false,
            not_in_downtime: false
        };

        if (doReload) {
            this.loadServices();
        }
    }


    // Generic callback whenever a mass action (like delete all) has been finished
    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadServices();
        }
    }

    public resetChecktime() {
        this.selectedItems = [];
        let selectedServiceUuids: string[] = this.SelectionServiceService.getSelectedItems().map((item) => item.service_description);

        const commands: ServiceResetItem[] = [];

        this.services?.servicestatus.forEach((item) => {
            if (typeof (item.service_description) === "undefined") {
                return;
            }
            if (!selectedServiceUuids.includes(item.service_description)) {
                return;
            }

            commands.push({
                command: ExternalCommandsEnum.rescheduleService,
                hostUuid: item.hostname,
                serviceUuid: item.service_description,
            });

        });

        if (commands.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        commands.forEach(command => {
            this.subscriptions.add(this.ExternalCommandsService.reset_check_time(command).subscribe((result) => {
            }));
        });

        const title = this.TranslocoService.translate('Reschedule');
        const msg: string = this.TranslocoService.translate('Commands added successfully to queue');

        this.notyService.genericSuccess(msg, title);

        setTimeout(() => {
            this.onMassActionComplete(true);
        }, 5000);
    }

    public toggleDowntimeModal() {
        let items: ServiceDowntimeItem[] = [];
        items = this.SelectionServiceService.getSelectedItems().map((item): ServiceDowntimeItem => {
            return {
                command: ExternalCommandsEnum.submitServiceDowntime,
                hostUuid: item.hostname,
                serviceUuid: item.service_description,
                start: 0,
                end: 0,
                author: this.userFullname,
                comment: ''
            };
        });

        this.selectedItems = items;
        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }
        this.modalService.toggle({
            show: true,
            id: 'serviceMaintenanceModal',
        });
    }

    public acknowledgeStatus() {
        let items: ServiceAcknowledgeItem[] = [];
        items = this.SelectionServiceService.getSelectedItems().map((item): ServiceAcknowledgeItem => {
            return {
                command: ExternalCommandsEnum.submitServicestateAck,
                hostUuid: item.hostname,
                serviceUuid: item.service_description,
                author: this.userFullname,
                comment: '',
                notify: true,
                sticky: false
            };
        });
        this.selectedItems = items;
        if (items.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }
        this.modalService.toggle({
            show: true,
            id: 'serviceAcknowledgeModal',
        });
    }


    protected readonly String = String;
}
