import { inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../tokens/proxy-path.token';
import { Observable } from 'rxjs';


export interface ServiceDowntimeItem {
    command: string,
    hostUuid: string,
    serviceUuid: string,
    start: number,
    end: number,
    author: string,
    comment: string,
}

export interface HostDowntimeItem {
    command: string,
    hostUuid: string,
    start: number,
    end: number,
    author: string,
    comment: string,
    downtimetype: string | number
}

export interface HostAcknowledgeItem {
    command: string,
    hostUuid: string,
    hostAckType: 'hostOnly' | 'hostAndServices',
    sticky: boolean,
    notify: boolean,
    author: string,
    comment: string,
}

export interface ServiceAcknowledgeItem {
    command: string,
    hostUuid: string,
    serviceUuid: string,
    sticky: boolean,
    notify: boolean,
    author: string,
    comment: string,
}

export interface ServiceResetItem {
    command: string
    hostUuid: string
    serviceUuid: string
}

export interface HostRescheduleItem {
    command: string
    hostUuid: string
    type: 'hostOnly' | 'hostAndServices'
}

export interface HostProcessCheckResultItem {
    command: string,
    hostUuid: string,
    plugin_output: string,
    long_output: string,
    status_code: number,
    forceHardstate: boolean
    maxCheckAttempts: number
}

export interface ServiceProcessCheckResultItem {
    command: string,
    hostUuid: string,
    serviceUuid: string,
    plugin_output: string,
    long_output: string,
    status_code: number,
    forceHardstate: boolean,
    maxCheckAttempts: number,
}

// This can be used for hosts and services
export interface EnableOrDisableFlapDetectionItem {
    command: string,
    hostUuid: string,
    serviceUuid?: string,
    condition: 0 | 1, // 0 - disable, 1 - enable
}


type Commands =
    ServiceDowntimeItem[]
    | HostAcknowledgeItem[]
    | ServiceAcknowledgeItem[]
    | ServiceResetItem[]
    | HostRescheduleItem[]
    | HostDowntimeItem[]
    | HostProcessCheckResultItem[]
    | ServiceProcessCheckResultItem[]
    | EnableOrDisableFlapDetectionItem[]

@Injectable({
    providedIn: 'root'
})
export class ExternalCommandsService {

    private readonly http = inject(HttpClient);
    private readonly proxyPath = inject(PROXY_PATH);

    constructor() {
    }


    public schedule_maintenance(downtime: HostDowntimeItem | ServiceDowntimeItem): Observable<Object> {
        const proxyPath = this.proxyPath;

        let params: any = {};

        if ('downtimetype' in downtime) {
            // This is a host downtime
            params = {
                angular: true,
                type: 'host',
                host_uuid: downtime.hostUuid,
                comment: downtime.comment,
                include_services: Number(downtime.downtimetype) === 1,

                // New Angular start/end API
                start: downtime.start,
                end: downtime.end
            };
        }

        if ('serviceUuid' in downtime) {
            params = {
                angular: true,
                type: 'service',
                host_uuid: downtime.hostUuid,
                service_uuid: downtime.serviceUuid,
                comment: downtime.comment,
                include_services: false,

                // New Angular start/end API
                start: downtime.start,
                end: downtime.end
            };
        }

        return this.http.get<{ success: boolean }>(`${proxyPath}/externalcommands/schedule_maintenance.json`, {
            params: params
        });
    }

    public reset_check_time(item: HostRescheduleItem | ServiceResetItem): Observable<Object> {
        const proxyPath = this.proxyPath;


        // This is a host
        let include_services: boolean = false;
        if ('type' in item) {
            if (item.type === 'hostAndServices') {
                include_services = true;
            }
        }

        let params: any = {
            angular: true,
            type: 'host',
            host_uuid: item.hostUuid,
            include_services: include_services
        };


        if ('serviceUuid' in item) {
            // This is a service
            params = {
                angular: true,
                type: 'service',
                host_uuid: item.hostUuid,
                service_uuid: item.serviceUuid,
                include_services: false
            };
        }

        return this.http.get<{ success: boolean }>(`${proxyPath}/externalcommands/reset_check_time.json`, {
            params: params
        });
    }

    public acknowledge_status(item: HostAcknowledgeItem | ServiceAcknowledgeItem): Observable<Object> {
        const proxyPath = this.proxyPath;


        // This is a host
        let include_services: boolean = false;
        if ('hostAckType' in item) {
            if (item.hostAckType === 'hostAndServices') {
                include_services = true;
            }
        }

        let params: any = {
            angular: true,
            type: 'host',
            host_uuid: item.hostUuid,
            include_services: include_services,
            comment: item.comment,
            is_sticky: item.sticky
        };


        if ('serviceUuid' in item) {
            // This is a service
            params = {
                angular: true,
                type: 'service',
                host_uuid: item.hostUuid,
                service_uuid: item.serviceUuid,
                include_services: false,
                comment: item.comment,
                is_sticky: item.sticky
            };
        }

        return this.http.get<{ success: boolean }>(`${proxyPath}/externalcommands/acknowledge_status.json`, {
            params: params
        });
    }

    public process_check_result(item: HostProcessCheckResultItem | ServiceProcessCheckResultItem): Observable<Object> {
        const proxyPath = this.proxyPath;


        // This is a host
        let include_services: boolean = false;
        if ('hostAckType' in item) {
            if (item.hostAckType === 'hostAndServices') {
                include_services = true;
            }
        }

        let params: any = {
            angular: true,
            host_uuid: item.hostUuid,
            type: 'host',
            output: item.plugin_output,
            state: item.status_code,
            force_hardstate: item.forceHardstate,
            repetitions: item.maxCheckAttempts
        };


        if ('serviceUuid' in item) {
            // This is a service
            params = {
                angular: true,
                host_uuid: item.hostUuid,
                service_uuid: item.serviceUuid,
                type: 'service',
                output: item.plugin_output,
                state: item.status_code,
                force_hardstate: item.forceHardstate,
                repetitions: item.maxCheckAttempts
            };
        }

        return this.http.get<{ success: boolean }>(`${proxyPath}/externalcommands/process_check_result.json`, {
            params: params
        });
    }

    public flap_detection(item: EnableOrDisableFlapDetectionItem): Observable<Object> {
        const proxyPath = this.proxyPath;


        let params: any = {
            angular: true,
            host_uuid: item.hostUuid,
            type: 'host',
            enabled: item.condition
        };

        if (item.serviceUuid) {
            params = {
                angular: true,
                host_uuid: item.hostUuid,
                service_uuid: item.serviceUuid,
                type: 'service',
                enabled: item.condition
            };
        }

        return this.http.get<{ success: boolean }>(`${proxyPath}/externalcommands/flap_detection.json`, {
            params: params
        });
    }

}
